"""Metadata for drf-firebase-auth package."""
__title__ = 'drf-firebase-auth'
__version__ = '1.0.0dev1'
__description__ = (
    'Custom Django Rest Framework authentication backend for '
    'parsing Firebase uid tokens and storing as local users.'
)
__url__ = 'https://gitlab.com/garyburgmann/drf-firebase-auth'
__author__ = 'Gary Burgmann'
__author_email__ = 'garyburgmann@gmail.com'
__license__ = 'MIT'
__copyright__ = 'Copyright 2021 Gary Burgmann'

# Version synonym
VERSION = __version__
